#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''                                                                                                             
Author: xuwei                                        
Email: 18810079020@163.com                                 
File: run_on_time.py
Date: 2020/7/29 10:27 上午
'''

import time
import datetime

class RunOnTime(object):
    def __init__(self, hour=None, minute=None, sleep_minute=40):
        self.__hour = hour
        self.__minute = minute
        self.__sleep_minute = sleep_minute

    def runOnHour(self, fun, *args, **kwargs):
        while True:
            now = datetime.datetime.now()
            if now.hour == self.__hour:
                fun(*args, **kwargs)
            time.sleep(60 * self.__sleep_minute)


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''                                                                                                             
Author: xuwei                                        
Email: 18810079020@163.com                                 
File: data_to_str.py
Date: 2021/1/6 6:39 下午
'''

import json
import numpy
import datetime
import pandas as pd
from datetime import date
from .utils import CJsonEncoder


class DataTypeToStr(object):
    @classmethod
    def data_type_to_str(cls, obj):
        if obj != obj:
            return None
        elif isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, numpy.datetime64):
            return pd.to_datetime(str(obj)).strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime("%Y-%m-%d")
        elif isinstance(obj, numpy.integer) or isinstance(obj, numpy.floating) or isinstance(obj, int) \
                or isinstance(obj, float) or isinstance(obj, bytes):
            return str(obj)
        elif isinstance(obj, numpy.ndarray):
            return json.dumps(obj.tolist(), ensure_ascii=False, cls=CJsonEncoder)
        elif isinstance(obj, list):
            return json.dumps(obj, ensure_ascii=False, cls=CJsonEncoder)
        elif isinstance(obj, dict):
            return json.dumps(obj, ensure_ascii=False, cls=CJsonEncoder)
        else:
            return obj

    @classmethod
    def dict_values_type_etl_str(cls, dict_param):
        for key in dict_param:
            dict_param[key] = cls.data_type_to_str(dict_param[key])
        return dict_param

    @classmethod
    def list_dict_values_type_to_str(cls, list_dict):
        list_dict = list(map(cls.dict_values_type_etl_str, list_dict))
        return list_dict




from .config_log import config, log, init_log, catch_error_log # 配置文档读取和log日志以及捕获异常装饰器

from .cut_word import TextEtl # 文本处理

from .utils import get_url_uuid, drop_element_contain_in_str_list, load_json, dump_json, get_quote # 常用工具

from .email import EmailFactory, send_mail # 发邮件
from .wechat import WechatFactory # 发微信

from .etl_time import TimeEtl, now_time, TimeCost, format_time_cost, make_timestamp, sleep # 时间相关

from .ReParse import Parse # 正则相关

from .run_on_time import RunOnTime # 指定时间运行程序

from .queue import RabbitProducer, RabbitConsumer  # 消息队列，生产者和消费者
from .queue2 import RabbitConsumer2 # 消息队列，消费者


# 数据库操作相关
from .es import BaseSearcher, BaseIndexer, getNerByEs # es model操作
from .elasticsearch_op import EsOp, get_es_client # es直接操作

from .redis_operate import RedisOp, get_redis_client # redis操作

from .mysql_op import UpdateTable, MysqlOp # mysql操作
from .db_op import DB  # mysql操作，实现了mysql连接池，长链接用

from .sqlserver_op import SqlServerOp # sqlserver操作

from .mongo_op import MongoOp  # mongo操作

from .hive_op import HiveOp  # hive操作

from .neo4j_op import Neo4jOp # neo4j操作

from .plt_graph import series_plot_graph # pandas的series画图

from .spider_op import SpiderOp # 爬虫selenium

# 多进程和线程相关
from .multi_process_thread import multi_process_map, multi_thread_map, multi_thread_return, multi_thread_return_funs_list

# uuid相关
# from .url_uuid import get_url_uuid, get_url_uuid_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Author: xuwei
Email: 18810079020@163.com
File: sqlserver_op.py
Date: 2020/8/18 3:36 下午
'''

import pandas as pd
from .config_log import config
from .utils import MyException

'''
## mac 使用sqlServer

- 1、先准备环境

        参考网址： https://www.microsoft.com/en-us/sql-server/developer-get-started/python/mac/?rtc=1
        brew tap microsoft/mssql-release https://github.com/Microsoft/homebrew-mssql-release
        brew update
        HOMEBREW_NO_ENV_FILTERING=1 ACCEPT_EULA=Y brew install msodbcsql17 mssql-tools
        sudo ln -s /usr/local/etc/odbcinst.ini /etc/odbcinst.ini
        sudo ln -s /usr/local/etc/odbc.ini /etc/odbc.ini
        
'''

class ConMysql(object):
    def __init__(self, label=None, host=None, port=None, user=None, password=None, db=None):
        self.__con_entity = dict()
        if label:
            self._host = config(label, 'host')
            self._port = int(config(label, 'port'))
            self._user = config(label, 'user')
            self._password = config(label, 'pass')
            self._db_name = db if db else config(label, 'db')
        else:
            self._host = host
            self._port = port
            self._user = user
            self._password = password
            self._db_name = db

        self.con_text = "DRIVER={driver};SERVER={host};DATABASE={db};UID={user};PWD={password}".format(
                    driver="{ODBC Driver 17 for SQL Server}",
                    host=self._host,
                    db=self._db_name,
                    user=self._user,
                    password=self._password,
                )

    def engine_db(self):
        _state = True
        _max_retries_count = 10  # 设置最大重试次数
        _conn_retries_count = 0  # 初始重试次数
        while _state and _conn_retries_count < _max_retries_count:
            try:
                import pyodbc
                engine = pyodbc.connect(self.con_text)
                return engine
            except:
                _conn_retries_count += 1
        raise MyException('%s-engine_db数据库连接失败' % self._host)

    def con_db(self):
        _state = True
        _max_retries_count = 10  # 设置最大重试次数
        _conn_retries_count = 0  # 初始重试次数
        while _state and _conn_retries_count < _max_retries_count:
            try:
                import pyodbc
                con = pyodbc.connect(self.con_text)
                return con
            except:
                _conn_retries_count += 1
        raise MyException('%s-con_db数据库连接失败' % self._host)


class SqlServerOp(ConMysql):
    def get_tuple_sql(self, sql, close_con=True):
        con = self.con_db()
        try:
            with con.cursor() as cursor:
                cursor.execute(sql)
                results = cursor.fetchall()
                return results
        except Exception as e:
            raise e
        finally:
            if close_con:
                con.close()

    def get_dict_sql(self, sql, close_con=True):
        con = self.con_db()
        try:
            with con.cursor() as cursor:
                cursor.execute(sql)
                list_tuple = cursor.fetchall()
                columns = [column [0] for column in cursor.description]
                result = []
                for obj_tuple in list_tuple:
                    item = dict()
                    for obj in zip(columns, obj_tuple):
                        item[obj[0]] = obj[1]
                    result.append(item)
                return result
        except Exception as e:
            raise e
        finally:
            if close_con:
                con.close()

    def get_df_sql(self, sql):
        try:
            df = pd.read_sql(sql, self.engine_db(), coerce_float=False)
            return df
        except Exception as e:
            raise e
        finally:
            self.engine_db().close()

    def get_df_sql_loop(self, sql, num):
        try:
            df_loop = pd.read_sql(sql, self.engine_db(), coerce_float=False, chunksize=num)
            return df_loop
        except Exception as e:
            raise e
        finally:
            self.engine_db().close()



import os
from setuptools import setup

# 根据es版本安装python es库
# pymongo
# "elasticsearch-dsl>=6.3.1",
# "elasticsearch-dsl>=7.2.0",

requirements = [
    'pika==0.12.0',
    'numpy==1.16.4',
    'shortuuid==0.5.0',
    'py2neo==2020.0.0',
    'threadpool==1.3.2',
    'redis==3.3.11',
    'elasticsearch_dsl==7.3.0',
    'matplotlib==3.1.3',
    'jieba==0.42.1',
    'wheel==0.34.2',
    'PyMySQL==0.9.3',
    'pypinyin==0.37.0',
    'selenium==3.141.0',
    'elasticsearch==7.5.1',
    'pymongo==3.10.1',
    'pandas==1.1.1',
    'pyodbc==4.0.30',
    'PyHive==0.6.2',
    'requests==2.22.0',
    'beautifulsoup4==4.9.3',
    'DBUtils==2.0',
]

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='xwtools',
    version='1.0.4',
    packages=[
        "xwtools",
    ],
    license='BSD License',  # example license
    description='xwtools',
    long_description='这是一个通用的python工具包，帮助你快速的开发项目',
    install_requires=requirements,
    long_description_content_type="text/markdown",
    url='https://github.com/xulehexuwei',
    author='xuwei',
    author_email='15200813194@163.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',  # replace "X.Y" as appropriate
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)




import xwtools as xt

def test_log_info():
    xt.init_log(file_path='./log', module='test')
    xt.log.info('logxxxxx')

    try:
        a = 1 / 0
    except Exception as e:
        xt.log.exception(e)


def test_config():
    print(xt.config('db_stock', 'dbms'))
    print(xt.config('database'))
    print(xt.config('log_level'))


def test_db():
    db = xt.MysqlOp(label='msyql', db='test')
    df = db.get_df_sql("sql")
    query_list = db.get_dict_sql("sql")
    db.to_sql_write_data(df, 'table_name')
    db.execute_many_write_data(df, 'table_name')
    db.operate_sql("sql")


    db = xt.SqlServerOp()


    db = xt.RedisOp(label='', db=2)
    db.set()
    db.redis_hget()


    db =xt.MongoOp()

    db = xt.Neo4jOp()

    db = xt.HiveOp()

    db = xt.EsOp()

    esSearch = xt.BaseSearcher()
    total, query_list = esSearch.search_by_multi_bool()


def test_spider():
    spider_op = xt.SpiderOp()


def test_multi_thread():
    xt.multi_thread_map()

    xt.multi_process_map()


xt.init_log('./log', module='test_raise_exception')
@xt.catch_error_log
def test_raise_exception():
    return 1 / 0


def test_encrypt():
    key = 'sdai0000'
    sk = xt.Encryption.encrypt_key(key)
    print('\n')
    print('encrypt code: {}'.format(sk))
    code = xt.Encryption.decrypt_key(sk)
    print(code)
    print(code == key)


def test_encrypt_dict():
    from xwtools import Encryption
    config_map = {'user': 'sdai', 'pass': 'sddddd222', 'host': '192.168.1.1', 'port': '32'}
    est = Encryption.encryt_dict(config_map)
    print(est)
    st = Encryption.decrypt_dict(est)
    print(st)
    est['port'] = 32
    print(est)
    print(Encryption.decrypt_dict(est))

#
# def consume_message(msg):
#     print(msg)
#     print('deal one')
#     # xt.log_info('consumer ' + str(os.getpid()) + ' msg=' + msg)
#     # time.sleep(2)
#
#
# def test_rabbit_produce():
#     producer = xt.RabbitProducer()
#     for i in range(1, 10):
#         producer.send('test1', 'helloo=%d' % i)
#
#
# def test_kafka_consumer():
#     consumer = xt.RabbitConsumer(topic='test3',
#                                  config_info={'user': 'admin', 'password': 'sense_mq@2018', 'host': '52.82.30.135',
#                                               'port': 5672})
#     consumer.execute(consume_message)
#
#
# def function_cal(body):
#     import json
#     msg = json.loads(body)
#     print(msg['msg_header']['msg_idx'])
#
#
# def test_rabbit_produce1():
#     producer = xt.RabbitProducer(label='rabbit')
#     producer.send('test1', 'helloo=%d' % 1)
#
#
# def test_rabbit_consume():
#     consumer = xt.RabbitConsumer('test1', label='rabbit')
#     consumer.execute_safely(rabbit_consume, prefetch_count=1, no_ack=False)
#
#
# def rabbit_consume(body):
#     print(body)
#
#
# def function_test():
#     consumer = xt.RabbitConsumer('cloud_data_pdf_test', 'rabbit_cloud')
#     consumer.execute_safely(function_cal)

#
# def test_mq_fanout_produce():
#     from xwtools import RabbitProducer
#     import json
#     produce = RabbitProducer('rabbit_88')
#     for _k in range(1, 20):
#         st = {'index': _k}
#         produce.send_safely('alg_test_exc', json.dumps(st), exchange_type='fanout')
#         print('send one {} sleep 10s'.format(_k))
#
#
# def test_mq_fanout_consumer():
#     def handler(body):
#         print(body)
#         st = xt.load_json(body)
#         print(st)
#
#     consumer = xt.RabbitConsumer(topic='alg_dealed_result2', label='rabbit_88', heartbeat=300)
#     consumer.execute_safely(handler, exchange='alg_test_exc')
#
#
# def test_wechat():
#     from xwtools import WechatFactory
#     wechat = WechatFactory('wechat')
#     wechat.send_message('111', [])
#
#
# def test_email():
#     from xwtools import EmailFactory
#     email = EmailFactory('email')
#     email.send_message('瞅你咋地', '你瞅啥')


if __name__ == '__main__':
    pass
    test_log_info()
    # test_config()
    # test_raise_exception()
